@@ -1,24 +1,23 @@
-/* A Bison parser, made by GNU Bison 2.3.  */
+
+/* A Bison parser, made by GNU Bison 2.4.1.  */
 
 /* Skeleton implementation for Bison's Yacc-like parsers in C
-
-   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
+   
+      Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
    Free Software Foundation, Inc.
-
-   This program is free software; you can redistribute it and/or modify
+   
+   This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2, or (at your option)
-   any later version.
-
+   the Free Software Foundation, either version 3 of the License, or
+   (at your option) any later version.
+   
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
-
+   
    You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301, USA.  */
+   along with this program.  If not, see <http://www.gnu.org/licenses/>.  */
 
 /* As a special exception, you may create a larger work that contains
    part or all of the Bison parser skeleton and distribute that work
@@ -29,7 +28,7 @@
    special exception, which will cause the skeleton and the resulting
    Bison output files to be licensed under the GNU General Public
    License without this special exception.
-
+   
    This special exception was added by the Free Software Foundation in
    version 2.2 of Bison.  */
 
@@ -47,7 +46,7 @@
 #define YYBISON 1
 
 /* Bison version.  */
-#define YYBISON_VERSION "2.3"
+#define YYBISON_VERSION "2.4.1"
 
 /* Skeleton name.  */
 #define YYSKELETON_NAME "yacc.c"
@@ -55,70 +54,29 @@
 /* Pure parsers.  */
 #define YYPURE 1
 
+/* Push parsers.  */
+#define YYPUSH 0
+
+/* Pull parsers.  */
+#define YYPULL 1
+
 /* Using locations.  */
 #define YYLSP_NEEDED 0
 
 /* Substitute the variable and function names.  */
-#define yyparse unitsparse
-#define yylex   unitslex
-#define yyerror unitserror
-#define yylval  unitslval
-#define yychar  unitschar
-#define yydebug unitsdebug
-#define yynerrs unitsnerrs
-
-
-/* Tokens.  */
-#ifndef YYTOKENTYPE
-# define YYTOKENTYPE
-   /* Put the tokens into the symbol table, so that GDB and other debuggers
-      know about them.  */
-   enum yytokentype {
-     REAL = 258,
-     UNIT = 259,
-     RFUNC = 260,
-     UFUNC = 261,
-     EXPONENT = 262,
-     MULTIPLY = 263,
-     MULTSTAR = 264,
-     DIVIDE = 265,
-     NUMDIV = 266,
-     SQRT = 267,
-     CUBEROOT = 268,
-     MULTMINUS = 269,
-     EOL = 270,
-     FUNCINV = 271,
-     SCANERROR = 272,
-     MINUS = 273,
-     ADD = 274,
-     UNARY = 275
-   };
-#endif
-/* Tokens.  */
-#define REAL 258
-#define UNIT 259
-#define RFUNC 260
-#define UFUNC 261
-#define EXPONENT 262
-#define MULTIPLY 263
-#define MULTSTAR 264
-#define DIVIDE 265
-#define NUMDIV 266
-#define SQRT 267
-#define CUBEROOT 268
-#define MULTMINUS 269
-#define EOL 270
-#define FUNCINV 271
-#define SCANERROR 272
-#define MINUS 273
-#define ADD 274
-#define UNARY 275
-
-
+#define yyparse         unitsparse
+#define yylex           unitslex
+#define yyerror         unitserror
+#define yylval          unitslval
+#define yychar          unitschar
+#define yydebug         unitsdebug
+#define yynerrs         unitsnerrs
 
 
 /* Copy the first part of user declarations.  */
-#line 23 "parse.y"
+
+/* Line 189 of yacc.c  */
+#line 24 "parse.y"
 
 
 #define YYPARSE_PARAM comm
@@ -128,6 +86,8 @@
 
 #include "units.h"
 
+int yylex();
+void yyerror(char *);
 
 static int err;  /* value used by parser to store return values */
 
@@ -213,7 +173,7 @@ funcunit(struct unittype *theunit, struct function *fun)
     if (err)
       return err;
   } else if (fun->type==ANGLEOUT || fun->type == DIMENSIONLESS) {
-    if (err=unit2num(theunit))
+    if ((err=unit2num(theunit)))
       return err;
     
   } else 
@@ -232,6 +192,9 @@ funcunit(struct unittype *theunit, struct function *fun)
 
 
 
+/* Line 189 of yacc.c  */
+#line 197 "parse.tab.c"
+
 /* Enabling traces.  */
 #ifndef YYDEBUG
 # define YYDEBUG 0
@@ -250,31 +213,65 @@ funcunit(struct unittype *theunit, struct function *fun)
 # define YYTOKEN_TABLE 0
 #endif
 
+
+/* Tokens.  */
+#ifndef YYTOKENTYPE
+# define YYTOKENTYPE
+   /* Put the tokens into the symbol table, so that GDB and other debuggers
+      know about them.  */
+   enum yytokentype {
+     REAL = 258,
+     UNIT = 259,
+     RFUNC = 260,
+     UFUNC = 261,
+     EXPONENT = 262,
+     MULTIPLY = 263,
+     MULTSTAR = 264,
+     DIVIDE = 265,
+     NUMDIV = 266,
+     SQRT = 267,
+     CUBEROOT = 268,
+     MULTMINUS = 269,
+     EOL = 270,
+     FUNCINV = 271,
+     SCANERROR = 272,
+     MINUS = 273,
+     ADD = 274,
+     UNARY = 275
+   };
+#endif
+
+
+
 #if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
 typedef union YYSTYPE
-#line 139 "parse.y"
 {
+
+/* Line 214 of yacc.c  */
+#line 142 "parse.y"
+
   double number;
   int integer;
   struct unittype *utype;
   struct function *dfunc;
   struct func *ufunc;
-}
-/* Line 187 of yacc.c.  */
-#line 265 "parse.tab.c"
-	YYSTYPE;
+
+
+
+/* Line 214 of yacc.c  */
+#line 263 "parse.tab.c"
+} YYSTYPE;
+# define YYSTYPE_IS_TRIVIAL 1
 # define yystype YYSTYPE /* obsolescent; will be withdrawn */
 # define YYSTYPE_IS_DECLARED 1
-# define YYSTYPE_IS_TRIVIAL 1
 #endif
 
 
-
 /* Copy the second part of user declarations.  */
 
 
-/* Line 216 of yacc.c.  */
-#line 278 "parse.tab.c"
+/* Line 264 of yacc.c  */
+#line 275 "parse.tab.c"
 
 #ifdef short
 # undef short
@@ -349,14 +346,14 @@ typedef short int yytype_int16;
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static int
-YYID (int i)
+YYID (int yyi)
 #else
 static int
-YYID (i)
-    int i;
+YYID (yyi)
+    int yyi;
 #endif
 {
-  return i;
+  return yyi;
 }
 #endif
 
@@ -437,9 +434,9 @@ void free (void *); /* INFRINGES ON USER NAME SPACE */
 /* A type that is properly aligned for any stack member.  */
 union yyalloc
 {
-  yytype_int16 yyss;
-  YYSTYPE yyvs;
-  };
+  yytype_int16 yyss_alloc;
+  YYSTYPE yyvs_alloc;
+};
 
 /* The size of the maximum gap between one aligned stack and the next.  */
 # define YYSTACK_GAP_MAXIMUM (sizeof (union yyalloc) - 1)
@@ -473,12 +470,12 @@ union yyalloc
    elements in the stack, and YYPTR gives the new location of the
    stack.  Advance YYPTR to a properly aligned location for the next
    stack.  */
-# define YYSTACK_RELOCATE(Stack)					\
+# define YYSTACK_RELOCATE(Stack_alloc, Stack)				\
     do									\
       {									\
 	YYSIZE_T yynewbytes;						\
-	YYCOPY (&yyptr->Stack, Stack, yysize);				\
-	Stack = &yyptr->Stack;						\
+	YYCOPY (&yyptr->Stack_alloc, Stack, yysize);			\
+	Stack = &yyptr->Stack_alloc;					\
 	yynewbytes = yystacksize * sizeof (*Stack) + YYSTACK_GAP_MAXIMUM; \
 	yyptr += yynewbytes / sizeof (*yyptr);				\
       }									\
@@ -569,10 +566,10 @@ static const yytype_int8 yyrhs[] =
 /* YYRLINE[YYN] -- source line where rule number YYN was defined.  */
 static const yytype_uint8 yyrline[] =
 {
-       0,   181,   181,   182,   183,   186,   187,   190,   191,   192,
-     193,   194,   196,   197,   198,   201,   202,   205,   211,   212,
-     213,   214,   215,   216,   217,   218,   219,   220,   221,   222,
-     225,   228
+       0,   184,   184,   185,   186,   189,   190,   193,   194,   195,
+     196,   197,   199,   200,   201,   204,   205,   208,   214,   215,
+     216,   217,   218,   219,   220,   221,   222,   223,   224,   225,
+     228,   231
 };
 #endif
 
@@ -895,17 +892,20 @@ yy_symbol_print (yyoutput, yytype, yyvaluep)
 #if (defined __STDC__ || defined __C99__FUNC__ \
      || defined __cplusplus || defined _MSC_VER)
 static void
-yy_stack_print (yytype_int16 *bottom, yytype_int16 *top)
+yy_stack_print (yytype_int16 *yybottom, yytype_int16 *yytop)
 #else
 static void
-yy_stack_print (bottom, top)
-    yytype_int16 *bottom;
-    yytype_int16 *top;
+yy_stack_print (yybottom, yytop)
+    yytype_int16 *yybottom;
+    yytype_int16 *yytop;
 #endif
 {
   YYFPRINTF (stderr, "Stack now");
-  for (; bottom <= top; ++bottom)
-    YYFPRINTF (stderr, " %d", *bottom);
+  for (; yybottom <= yytop; yybottom++)
+    {
+      int yybot = *yybottom;
+      YYFPRINTF (stderr, " %d", yybot);
+    }
   YYFPRINTF (stderr, "\n");
 }
 
@@ -939,11 +939,11 @@ yy_reduce_print (yyvsp, yyrule)
   /* The symbols being reduced.  */
   for (yyi = 0; yyi < yynrhs; yyi++)
     {
-      fprintf (stderr, "   $%d = ", yyi + 1);
+      YYFPRINTF (stderr, "   $%d = ", yyi + 1);
       yy_symbol_print (stderr, yyrhs[yyprhs[yyrule] + yyi],
 		       &(yyvsp[(yyi + 1) - (yynrhs)])
 		       		       );
-      fprintf (stderr, "\n");
+      YYFPRINTF (stderr, "\n");
     }
 }
 
@@ -1223,10 +1223,8 @@ yydestruct (yymsg, yytype, yyvaluep)
 	break;
     }
 }
-
 
 /* Prevent warnings from -Wmissing-prototypes.  */
-
 #ifdef YYPARSE_PARAM
 #if defined __STDC__ || defined __cplusplus
 int yyparse (void *YYPARSE_PARAM);
@@ -1245,10 +1243,9 @@ int yyparse ();
 
 
 
-
-/*----------.
-| yyparse.  |
-`----------*/
+/*-------------------------.
+| yyparse or yypush_parse.  |
+`-------------------------*/
 
 #ifdef YYPARSE_PARAM
 #if (defined __STDC__ || defined __C99__FUNC__ \
@@ -1272,22 +1269,46 @@ yyparse ()
 #endif
 #endif
 {
-  /* The look-ahead symbol.  */
+/* The lookahead symbol.  */
 int yychar;
 
-/* The semantic value of the look-ahead symbol.  */
+/* The semantic value of the lookahead symbol.  */
 YYSTYPE yylval;
 
-/* Number of syntax errors so far.  */
-int yynerrs;
+    /* Number of syntax errors so far.  */
+    int yynerrs;
+
+    int yystate;
+    /* Number of tokens to shift before error messages enabled.  */
+    int yyerrstatus;
+
+    /* The stacks and their tools:
+       `yyss': related to states.
+       `yyvs': related to semantic values.
+
+       Refer to the stacks thru separate pointers, to allow yyoverflow
+       to reallocate them elsewhere.  */
+
+    /* The state stack.  */
+    yytype_int16 yyssa[YYINITDEPTH];
+    yytype_int16 *yyss;
+    yytype_int16 *yyssp;
+
+    /* The semantic value stack.  */
+    YYSTYPE yyvsa[YYINITDEPTH];
+    YYSTYPE *yyvs;
+    YYSTYPE *yyvsp;
+
+    YYSIZE_T yystacksize;
 
-  int yystate;
   int yyn;
   int yyresult;
-  /* Number of tokens to shift before error messages enabled.  */
-  int yyerrstatus;
-  /* Look-ahead token as an internal (translated) token number.  */
-  int yytoken = 0;
+  /* Lookahead token as an internal (translated) token number.  */
+  int yytoken;
+  /* The variables used to return semantic value and location from the
+     action routines.  */
+  YYSTYPE yyval;
+
 #if YYERROR_VERBOSE
   /* Buffer for error messages, and its allocated size.  */
   char yymsgbuf[128];
@@ -1295,51 +1316,28 @@ int yynerrs;
   YYSIZE_T yymsg_alloc = sizeof yymsgbuf;
 #endif
 
-  /* Three stacks and their tools:
-     `yyss': related to states,
-     `yyvs': related to semantic values,
-     `yyls': related to locations.
-
-     Refer to the stacks thru separate pointers, to allow yyoverflow
-     to reallocate them elsewhere.  */
-
-  /* The state stack.  */
-  yytype_int16 yyssa[YYINITDEPTH];
-  yytype_int16 *yyss = yyssa;
-  yytype_int16 *yyssp;
-
-  /* The semantic value stack.  */
-  YYSTYPE yyvsa[YYINITDEPTH];
-  YYSTYPE *yyvs = yyvsa;
-  YYSTYPE *yyvsp;
-
-
-
 #define YYPOPSTACK(N)   (yyvsp -= (N), yyssp -= (N))
 
-  YYSIZE_T yystacksize = YYINITDEPTH;
-
-  /* The variables used to return semantic value and location from the
-     action routines.  */
-  YYSTYPE yyval;
-
-
   /* The number of symbols on the RHS of the reduced rule.
      Keep to zero when no symbol should be popped.  */
   int yylen = 0;
 
+  yytoken = 0;
+  yyss = yyssa;
+  yyvs = yyvsa;
+  yystacksize = YYINITDEPTH;
+
   YYDPRINTF ((stderr, "Starting parse\n"));
 
   yystate = 0;
   yyerrstatus = 0;
   yynerrs = 0;
-  yychar = YYEMPTY;		/* Cause a token to be read.  */
+  yychar = YYEMPTY; /* Cause a token to be read.  */
 
   /* Initialize stack pointers.
      Waste one element of value and location stack
      so that they stay on the same level as the state stack.
      The wasted elements are never initialized.  */
-
   yyssp = yyss;
   yyvsp = yyvs;
 
@@ -1369,7 +1367,6 @@ int yynerrs;
 	YYSTYPE *yyvs1 = yyvs;
 	yytype_int16 *yyss1 = yyss;
 
-
 	/* Each stack pointer address is followed by the size of the
 	   data in use in that stack, in bytes.  This used to be a
 	   conditional around just the two extra args, but that might
@@ -1377,7 +1374,6 @@ int yynerrs;
 	yyoverflow (YY_("memory exhausted"),
 		    &yyss1, yysize * sizeof (*yyssp),
 		    &yyvs1, yysize * sizeof (*yyvsp),
-
 		    &yystacksize);
 
 	yyss = yyss1;
@@ -1400,9 +1396,8 @@ int yynerrs;
 	  (union yyalloc *) YYSTACK_ALLOC (YYSTACK_BYTES (yystacksize));
 	if (! yyptr)
 	  goto yyexhaustedlab;
-	YYSTACK_RELOCATE (yyss);
-	YYSTACK_RELOCATE (yyvs);
-
+	YYSTACK_RELOCATE (yyss_alloc, yyss);
+	YYSTACK_RELOCATE (yyvs_alloc, yyvs);
 #  undef YYSTACK_RELOCATE
 	if (yyss1 != yyssa)
 	  YYSTACK_FREE (yyss1);
@@ -1413,7 +1408,6 @@ int yynerrs;
       yyssp = yyss + yysize - 1;
       yyvsp = yyvs + yysize - 1;
 
-
       YYDPRINTF ((stderr, "Stack size increased to %lu\n",
 		  (unsigned long int) yystacksize));
 
@@ -1423,6 +1417,9 @@ int yynerrs;
 
   YYDPRINTF ((stderr, "Entering state %d\n", yystate));
 
+  if (yystate == YYFINAL)
+    YYACCEPT;
+
   goto yybackup;
 
 /*-----------.
@@ -1431,16 +1428,16 @@ int yynerrs;
 yybackup:
 
   /* Do appropriate processing given the current state.  Read a
-     look-ahead token if we need one and don't already have one.  */
+     lookahead token if we need one and don't already have one.  */
 
-  /* First try to decide what to do without reference to look-ahead token.  */
+  /* First try to decide what to do without reference to lookahead token.  */
   yyn = yypact[yystate];
   if (yyn == YYPACT_NINF)
     goto yydefault;
 
-  /* Not known => get a look-ahead token if don't already have one.  */
+  /* Not known => get a lookahead token if don't already have one.  */
 
-  /* YYCHAR is either YYEMPTY or YYEOF or a valid look-ahead symbol.  */
+  /* YYCHAR is either YYEMPTY or YYEOF or a valid lookahead symbol.  */
   if (yychar == YYEMPTY)
     {
       YYDPRINTF ((stderr, "Reading a token: "));
@@ -1472,20 +1469,16 @@ yybackup:
       goto yyreduce;
     }
 
-  if (yyn == YYFINAL)
-    YYACCEPT;
-
   /* Count tokens shifted since error; after three, turn off error
      status.  */
   if (yyerrstatus)
     yyerrstatus--;
 
-  /* Shift the look-ahead token.  */
+  /* Shift the lookahead token.  */
   YY_SYMBOL_PRINT ("Shifting", yytoken, &yylval, &yylloc);
 
-  /* Discard the shifted token unless it is eof.  */
-  if (yychar != YYEOF)
-    yychar = YYEMPTY;
+  /* Discard the shifted token.  */
+  yychar = YYEMPTY;
 
   yystate = yyn;
   *++yyvsp = yylval;
@@ -1525,161 +1518,222 @@ yyreduce:
   switch (yyn)
     {
         case 2:
-#line 181 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 184 "parse.y"
     { COMM->result = makenumunit(1,&err); CHECK; YYACCEPT; ;}
     break;
 
   case 3:
-#line 182 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 185 "parse.y"
     { COMM->result = (yyvsp[(1) - (2)].utype); YYACCEPT; ;}
     break;
 
   case 4:
-#line 183 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 186 "parse.y"
     { YYABORT; ;}
     break;
 
   case 5:
-#line 186 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 189 "parse.y"
     {(yyval.utype) = (yyvsp[(1) - (1)].utype);;}
     break;
 
   case 6:
-#line 187 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 190 "parse.y"
     { invertunit((yyvsp[(2) - (2)].utype)); (yyval.utype)=(yyvsp[(2) - (2)].utype);;}
     break;
 
   case 7:
-#line 190 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 193 "parse.y"
     { (yyval.utype) = (yyvsp[(1) - (1)].utype); ;}
     break;
 
   case 8:
-#line 191 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 194 "parse.y"
     { (yyval.utype) = (yyvsp[(2) - (2)].utype); (yyval.utype)->factor *= -1; ;}
     break;
 
   case 9:
-#line 192 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 195 "parse.y"
     { (yyval.utype) = (yyvsp[(2) - (2)].utype); (yyval.utype)->factor *= -1; ;}
     break;
 
   case 10:
-#line 193 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 196 "parse.y"
     { err = addunit((yyvsp[(1) - (3)].utype),(yyvsp[(3) - (3)].utype)); CHECK; (yyval.utype)=(yyvsp[(1) - (3)].utype);;}
     break;
 
   case 11:
-#line 194 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 197 "parse.y"
     { (yyvsp[(3) - (3)].utype)->factor *= -1; err = addunit((yyvsp[(1) - (3)].utype),(yyvsp[(3) - (3)].utype)); 
                                          CHECK; (yyval.utype)=(yyvsp[(1) - (3)].utype);;}
     break;
 
   case 12:
-#line 196 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 199 "parse.y"
     { err = divunit((yyvsp[(1) - (3)].utype), (yyvsp[(3) - (3)].utype)); CHECK; (yyval.utype)=(yyvsp[(1) - (3)].utype);;}
     break;
 
   case 13:
-#line 197 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 200 "parse.y"
     { err = multunit((yyvsp[(1) - (3)].utype),(yyvsp[(3) - (3)].utype)); CHECK; (yyval.utype)=(yyvsp[(1) - (3)].utype);;}
     break;
 
   case 14:
-#line 198 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 201 "parse.y"
     { err = multunit((yyvsp[(1) - (3)].utype),(yyvsp[(3) - (3)].utype)); CHECK; (yyval.utype)=(yyvsp[(1) - (3)].utype);;}
     break;
 
   case 15:
-#line 201 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 204 "parse.y"
     { (yyval.number) = (yyvsp[(1) - (1)].number);         ;}
     break;
 
   case 16:
-#line 202 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 205 "parse.y"
     { (yyval.number) = (yyvsp[(1) - (3)].number) / (yyvsp[(3) - (3)].number);    ;}
     break;
 
   case 17:
-#line 205 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 208 "parse.y"
     { (yyval.utype) = (yyvsp[(2) - (3)].utype);  ;}
     break;
 
   case 18:
-#line 211 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 214 "parse.y"
     { (yyval.utype) = makenumunit((yyvsp[(1) - (1)].number),&err); CHECK;;}
     break;
 
   case 19:
-#line 212 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 215 "parse.y"
     { (yyval.utype) = (yyvsp[(1) - (1)].utype); ;}
     break;
 
   case 20:
-#line 213 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 216 "parse.y"
     { err = unitpower((yyvsp[(1) - (3)].utype),(yyvsp[(3) - (3)].utype)); CHECK; (yyval.utype)=(yyvsp[(1) - (3)].utype);;}
     break;
 
   case 21:
-#line 214 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 217 "parse.y"
     { err = multunit((yyvsp[(1) - (3)].utype),(yyvsp[(3) - (3)].utype)); CHECK; (yyval.utype)=(yyvsp[(1) - (3)].utype);;}
     break;
 
   case 22:
-#line 215 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 218 "parse.y"
     { err = multunit((yyvsp[(1) - (2)].utype),(yyvsp[(2) - (2)].utype)); CHECK; (yyval.utype)=(yyvsp[(1) - (2)].utype);;}
     break;
 
   case 23:
-#line 216 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 219 "parse.y"
     { (yyval.utype)=(yyvsp[(1) - (1)].utype); ;}
     break;
 
   case 24:
-#line 217 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 220 "parse.y"
     { err = rootunit((yyvsp[(2) - (2)].utype),2); CHECK; (yyval.utype)=(yyvsp[(2) - (2)].utype);;}
     break;
 
   case 25:
-#line 218 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 221 "parse.y"
     { err = rootunit((yyvsp[(2) - (2)].utype),3); CHECK; (yyval.utype)=(yyvsp[(2) - (2)].utype);;}
     break;
 
   case 26:
-#line 219 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 222 "parse.y"
     { err = funcunit((yyvsp[(2) - (2)].utype),(yyvsp[(1) - (2)].dfunc)); CHECK; (yyval.utype)=(yyvsp[(2) - (2)].utype);;}
     break;
 
   case 27:
-#line 220 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 223 "parse.y"
     { err = evalfunc((yyvsp[(2) - (2)].utype),(yyvsp[(1) - (2)].ufunc),0); CHECK; (yyval.utype)=(yyvsp[(2) - (2)].utype);;}
     break;
 
   case 28:
-#line 221 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 224 "parse.y"
     { err = evalfunc((yyvsp[(3) - (3)].utype),(yyvsp[(2) - (3)].ufunc),1); CHECK; (yyval.utype)=(yyvsp[(3) - (3)].utype);;}
     break;
 
   case 29:
-#line 223 "parse.y"
-    { (yyvsp[(4) - (4)].utype)->factor *= -1;
-				   err = unitpower((yyvsp[(1) - (4)].utype),(yyvsp[(4) - (4)].utype)); CHECK; (yyval.utype)=(yyvsp[(1) - (4)].utype);;}
-    break;
 
-  case 30:
+/* Line 1455 of yacc.c  */
 #line 226 "parse.y"
     { (yyvsp[(4) - (4)].utype)->factor *= -1;
 				   err = unitpower((yyvsp[(1) - (4)].utype),(yyvsp[(4) - (4)].utype)); CHECK; (yyval.utype)=(yyvsp[(1) - (4)].utype);;}
     break;
 
+  case 30:
+
+/* Line 1455 of yacc.c  */
+#line 229 "parse.y"
+    { (yyvsp[(4) - (4)].utype)->factor *= -1;
+				   err = unitpower((yyvsp[(1) - (4)].utype),(yyvsp[(4) - (4)].utype)); CHECK; (yyval.utype)=(yyvsp[(1) - (4)].utype);;}
+    break;
+
   case 31:
-#line 228 "parse.y"
+
+/* Line 1455 of yacc.c  */
+#line 231 "parse.y"
     { err = E_PARSEMEM; CHECK; ;}
     break;
 
 
-/* Line 1267 of yacc.c.  */
-#line 1683 "parse.tab.c"
+
+/* Line 1455 of yacc.c  */
+#line 1737 "parse.tab.c"
       default: break;
     }
   YY_SYMBOL_PRINT ("-> $$ =", yyr1[yyn], &yyval, &yyloc);
@@ -1690,7 +1744,6 @@ yyreduce:
 
   *++yyvsp = yyval;
 
-
   /* Now `shift' the result of the reduction.  Determine what state
      that goes to, based on the state we popped back to and the rule
      number reduced by.  */
@@ -1755,7 +1808,7 @@ yyerrlab:
 
   if (yyerrstatus == 3)
     {
-      /* If just tried and failed to reuse look-ahead token after an
+      /* If just tried and failed to reuse lookahead token after an
 	 error, discard it.  */
 
       if (yychar <= YYEOF)
@@ -1772,7 +1825,7 @@ yyerrlab:
 	}
     }
 
-  /* Else will try to reuse look-ahead token after shifting the error
+  /* Else will try to reuse lookahead token after shifting the error
      token.  */
   goto yyerrlab1;
 
@@ -1829,9 +1882,6 @@ yyerrlab1:
       YY_STACK_PRINT (yyss, yyssp);
     }
 
-  if (yyn == YYFINAL)
-    YYACCEPT;
-
   *++yyvsp = yylval;
 
 
@@ -1856,7 +1906,7 @@ yyabortlab:
   yyresult = 1;
   goto yyreturn;
 
-#ifndef yyoverflow
+#if !defined(yyoverflow) || YYERROR_VERBOSE
 /*-------------------------------------------------.
 | yyexhaustedlab -- memory exhaustion comes here.  |
 `-------------------------------------------------*/
@@ -1867,7 +1917,7 @@ yyexhaustedlab:
 #endif
 
 yyreturn:
-  if (yychar != YYEOF && yychar != YYEMPTY)
+  if (yychar != YYEMPTY)
      yydestruct ("Cleanup: discarding lookahead",
 		 yytoken, &yylval);
   /* Do not reclaim the symbols of the rule which action triggered
@@ -1893,7 +1943,9 @@ yyreturn:
 }
 
 
-#line 234 "parse.y"
+
+/* Line 1675 of yacc.c  */
+#line 237 "parse.y"
 
 
 #ifndef strchr
@@ -1908,40 +1960,40 @@ double strtod();
 
 
 struct function 
-  realfunctions[] = { "sin", sin,    ANGLEIN,
-                      "cos", cos,    ANGLEIN,
-                      "tan", tan,    ANGLEIN,
-                      "ln", log,     DIMENSIONLESS,
-                      "log", log10,  DIMENSIONLESS,
-                      "log2", logb2, DIMENSIONLESS,
-                      "exp", exp,    DIMENSIONLESS,
-                      "acos", acos,  ANGLEOUT,
-                      "atan", atan,  ANGLEOUT,
-                      "asin", asin,  ANGLEOUT,
-                      0, 0, 0};
+  realfunctions[] = { {"sin", sin,    ANGLEIN},
+                      {"cos", cos,    ANGLEIN},
+                      {"tan", tan,    ANGLEIN},
+                      {"ln", log,     DIMENSIONLESS},
+                      {"log", log10,  DIMENSIONLESS},
+                      {"log2", logb2, DIMENSIONLESS},
+                      {"exp", exp,    DIMENSIONLESS},
+                      {"acos", acos,  ANGLEOUT},
+                      {"atan", atan,  ANGLEOUT},
+                      {"asin", asin,  ANGLEOUT},
+                      {0, 0, 0}};
 
 struct {
   char op;
   int value;
-} optable[] = { '*', MULTIPLY,
-            	'/', DIVIDE, 
-            	'|', NUMDIV, 
-            	'+', ADD, 
-            	'(', '(', 
-            	')', ')', 
-            	'^', EXPONENT, 
-                '~', FUNCINV,
-            	0, 0 };
+} optable[] = { {'*', MULTIPLY},
+                {'/', DIVIDE},
+                {'|', NUMDIV},
+                {'+', ADD},
+                {'(', '('},
+                {')', ')'},
+                {'^', EXPONENT},
+                {'~', FUNCINV},
+                {0, 0}};
 
 struct {
   char *name;
   int value;
-} strtable[] = { "sqrt", SQRT, 
-                 "cuberoot", CUBEROOT, 
-                 "per" , DIVIDE, 
-                 0, 0 };
+} strtable[] = { {"sqrt", SQRT},
+                 {"cuberoot", CUBEROOT},
+                 {"per" , DIVIDE},
+                 {0, 0}};
 
-yylex(YYSTYPE *lvalp, struct commtype *comm)
+int yylex(YYSTYPE *lvalp, struct commtype *comm)
 {
   int length, count;
   struct unittype *output;
@@ -2077,7 +2129,7 @@ yylex(YYSTYPE *lvalp, struct commtype *comm)
 }
 
 
-yyerror(char *s){}
+void yyerror(char *s){}
 
 void
 freelist(int startunit)
