@@ -1,9 +1,9 @@
-#define VERSION "1.87"
+#define VERSION "1.88a"
 
 /*
  *  units, a program for units conversion
  *  Copyright (C) 1996, 1997, 1999, 2000, 2001, 2002, 2003, 2004, 2005, 2006,
- *                2007
+ *                2007, 2009
  *  Free Software Foundation, Inc
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -27,8 +27,15 @@
 #include<stdio.h>
 #include<signal.h>
 
+#ifdef SUPPORT_UTF8
+#  include <wchar.h>
+#  include <locale.h>
+#  include <langinfo.h>
+#endif
+
 #ifdef READLINE
 #  include <readline/readline.h>
+#  include <readline/history.h>
 #  define RVERSTR "with readline"
 #else
 #  define RVERSTR "without readline"
@@ -61,7 +68,8 @@
                                 /* doing unit comparisons. */
 
 char *pager;                    /* Actual pager (from PAGER environment var) */
-char *mylocale;                 /* Locale in effect (from LOCALE env var) */
+char *mylocale;                 /* Locale in effect (from LC_CTYPE or LANG) */
+int utf8mode;                   /* Activate UTF8 support */
 char *numformat = "%.8g";	/* printf format for numeric output */
 char *powerstring = "^";	/* Exponent character used in output */
 int quiet = 0;                  /* Flag for supressing prompting (-q option) */
@@ -389,6 +397,7 @@ readunits(char *file, FILE *errfile,
    struct func *funcentry;
    int wronglocale = 0;   /* If set then we are currently reading data */
    int inlocale = 0;      /* for the wrong locale so we should skip it */
+   int in_utf8 = 0;     /* If set we are reading utf8 data */
    locunitcount = 0;
    locprefixcount = 0;
    locfunccount  = 0;
@@ -407,6 +416,7 @@ readunits(char *file, FILE *errfile,
    while (!feof(unitfile)) {
       if (!fgetslong(&line, &linebufsize, unitfile, &linenum)) 
         break;
+
       if (*line == COMMANDCHAR) {         /* Process units.dat commands    */
         unitname = strtok(line+1, WHITE);
 	if (!strcmp(unitname,"locale")){ 
@@ -442,6 +452,29 @@ readunits(char *file, FILE *errfile,
 	  inlocale = 0;
 	  continue;
 	}
+        else if (!strcmp(unitname, "utf8")){
+          if (in_utf8){
+            if (errfile)
+              fprintf(errfile,"%s: nested utf8 not allowed, line %d of '%s'\n",
+                      progname, linenum, file);
+            goterr=1;
+          } else {
+            in_utf8 = 1;
+          }
+          continue;
+        } 
+        else if (!strcmp(unitname, "endutf8")){
+          if (!in_utf8){
+            if (errfile)
+              fprintf(errfile,"%s: unmatched !endutf8 on line %d of '%s'\n",
+                      progname, linenum, file);
+            goterr=1;
+          }
+          in_utf8 = 0;
+          continue;
+        }
+        if (in_utf8 && !utf8mode)
+          continue;
 	if (wronglocale)
 	  continue;
         if (!strcmp(unitname, "include")){
@@ -487,9 +520,11 @@ readunits(char *file, FILE *errfile,
 	}
 	continue;
       }	
+      if (in_utf8 && !utf8mode)
+        continue;
       if (wronglocale)
 	continue;
-      if (lineptr = strchr(line,COMMENTCHAR))
+      if ((lineptr = strchr(line,COMMENTCHAR)))
          *lineptr = 0;
       unitname = strtok(line, WHITE);
       if (!unitname || !*unitname) 
@@ -519,9 +554,9 @@ readunits(char *file, FILE *errfile,
   	     goterr=1;
 	     continue;
 	 }
-	 if (pfxptr = plookup(unitname)) {  /* already there: redefinition */
+	 if ((pfxptr = plookup(unitname))) {  /* already there: redefinition */
  	    goterr=1;
-            if (errfile)
+            if (errfile) {
 	      if (!strcmp(pfxptr->name, unitname))
 		fprintf(errfile,
    	        "%s: redefinition of prefix '%s-' on line %d of '%s' ignored.\n",
@@ -529,6 +564,7 @@ readunits(char *file, FILE *errfile,
 	      else
 	         fprintf(errfile, "%s: prefix '%s-' on line %d of '%s' is hidden by earlier definition of '%s-'.\n",
 		       progname, unitname, linenum, file, pfxptr->name);
+            }
 	    continue;
 	 } 
 
@@ -948,7 +984,7 @@ lookupunit(char *unit,int prefixok)
    struct prefixlist *pfxptr;
    struct unitlist *uptr;
 
-   if (uptr = ulookup(unit))
+   if ((uptr = ulookup(unit)))
       return uptr->value;
 
    if (strlen(unit)>2 && unit[strlen(unit) - 1] == 's') {
@@ -1278,7 +1314,7 @@ completereduce(struct unittype *unit)
 {
    int err;
 
-   if (err=reduceunit(unit))
+   if ((err=reduceunit(unit)))
      return err;
    sortunit(unit);
    cancelunit(unit);
@@ -1340,7 +1376,7 @@ unit2num(struct unittype *input)
   int err;
 
   initializeunit(&one);
-  if (err=completereduce(input))
+  if ((err=completereduce(input)))
     return err;
   if (compareunits(input,&one,ignore_nothing))
     return E_NOTANUMBER;
@@ -1404,13 +1440,14 @@ rootunit(struct unittype *inunit,int n)
    int err; 
 
    initializeunit(&outunit);
-   if (err=completereduce(inunit))
+   if ((err=completereduce(inunit)))
      return err;
-   if ((n & 1==0) && (inunit->factor<0)) return E_NOTROOT;
+   /* Even numbered root with negative number would be complex */
+   if ((n & 1)==0 && inunit->factor<0) return E_NOTROOT;
    outunit.factor = pow(inunit->factor,1.0/(double)n);
-   if (err = subunitroot(n, inunit->numerator, outunit.numerator))
+   if ((err = subunitroot(n, inunit->numerator, outunit.numerator)))
      return err;
-   if (err = subunitroot(n, inunit->denominator, outunit.denominator))
+   if ((err = subunitroot(n, inunit->denominator, outunit.denominator)))
      return err;
    freeunit(inunit);
    initializeunit(inunit);
@@ -1482,9 +1519,9 @@ addunit(struct unittype *unita, struct unittype *unitb)
 {
   int err;
 
-  if (err=completereduce(unita))
+  if ((err=completereduce(unita)))
     return err;
-  if (err=completereduce(unitb))
+  if ((err=completereduce(unitb)))
     return err;
   if (compareunits(unita,unitb,ignore_nothing))
     return E_BADSUM;
@@ -1723,7 +1760,7 @@ showanswer(char *havestr,struct unittype *have,
 {
    struct unittype invhave;
    int doingrec;  /* reciprocal conversion? */
-   char *sep, *right, *left;
+   char *sep = NULL, *right = NULL, *left = NULL;
 
    doingrec=0;
    havestr = removepadding(havestr);
@@ -1970,7 +2007,8 @@ addtolist(struct unittype *have, char *searchstring, char *rname, char *name, ch
 	       int *count, int searchtype)
 {
   struct unittype want;
-  int len,keepit;
+  int len = 0;
+  int keepit = 0;
 
   if (!name) 
     return;
@@ -1999,7 +2037,7 @@ addtolist(struct unittype *have, char *searchstring, char *rname, char *name, ch
     else
       (*list)[*count].def = def;
     (*count)++;
-    len = strlen(name);
+    len = strwidth(name);
     if (len>*maxnamelen)
 	  *maxnamelen = len;
   }
@@ -2084,7 +2122,7 @@ tryallunits(struct unittype *have, char *searchstring)
     outfile = stdout;
   for(i=0;i<count;i++){
     fputs(list[i].name,outfile);
-    for(j=strlen(list[i].name);j<=maxnamelen;j++)
+    for(j=strwidth(list[i].name);j<=maxnamelen;j++)
       putc(' ',outfile);
     fprintf(outfile, "%s\n",list[i].def);
   }
@@ -2243,7 +2281,7 @@ completeunits(char *text, int state)
       curunit = utab[uhash];
     }
     if (!curunit && !curprefix){
-      if (curprefix = plookup(text)){
+      if ((curprefix = plookup(text))){
         if (strlen(curprefix->name)>1 && strlen(curprefix->name)<strlen(text)){
           uhash = 0;
           curunit = utab[uhash];
@@ -2474,21 +2512,53 @@ processargs(int argc, char **argv, char **from, char **to)
 */
 
 
+
+#ifdef SUPPORT_UTF8
+int 
+strwidth(const char *str)
+{
+  wchar_t *widestr;
+  int len;
+
+  if (!utf8mode)
+    return strlen(str);
+  len = strlen(str)+1;
+  widestr = mymalloc(sizeof(wchar_t)*len, "(strwidth)");
+  len = mbsrtowcs(widestr, &str, len, NULL);
+
+  /* This checks for an invalid multibyte sequence.  I'm not 
+     sure what the right thing to do in this case is.*/
+  if (len==-1) return strlen(str);
+  return wcswidth(widestr, len);
+}
+#else
+#  define strwidth strlen
+#endif 
+
+int
+printspaces(int count)
+{
+  while((count--)>0) putchar(' ');
+}
  
 int 
 processunit(struct unittype *theunit, char *unitstr, char *prompt, int pointer)
 {
   char *errmsg;
   int errloc,err;
+  char savechar;
 
-  if (err=parseunit(theunit, unitstr, &errmsg, &errloc)){
+  if ((err=parseunit(theunit, unitstr, &errmsg, &errloc))){
     if (pointer){
       if (err!=E_UNKNOWNUNIT || !irreducible){
-	if (!quiet) {
-          while(*prompt++) putchar(' ');
-	}
-	if (errloc>0)  
-	  while(--errloc) putchar(' ');
+	if (!quiet) 
+          printspaces(strwidth(prompt));
+        if (errloc>0) {
+          savechar = unitstr[errloc];
+          unitstr[errloc] = 0;
+          printspaces(strwidth(unitstr)-1);
+          unitstr[errloc] = savechar;
+        }
 	printf("^\n");
       }
     }
@@ -2501,7 +2571,7 @@ processunit(struct unittype *theunit, char *unitstr, char *prompt, int pointer)
 
     return 1;
   }
-  if (err=completereduce(theunit)){
+  if ((err=completereduce(theunit))){
     printf("%s",errormsg[err]);
     if (err==E_UNKNOWNUNIT)
       printf(" '%s'", irreducible);
@@ -2584,11 +2654,11 @@ Typing 'search text' will show units whose names contain 'text'.\n\n",
              UNITMATCH);
       return 1;
     }
-    if (function = isfunction(str)){
+    if ((function = isfunction(str))){
       file = function->file;
       unitline = function->linenumber;
     }
-    else if (unit = ulookup(str)){
+    else if ((unit = ulookup(str))){
       unitline = unit->linenumber;
       file = unit->file;
     }
@@ -2608,8 +2678,43 @@ Typing 'search text' will show units whose names contain 'text'.\n\n",
   }
   return 0;
 }
-
  
+#ifdef SUPPORT_UTF8
+void
+checklocale()
+{
+  char *temp;
+  temp = setlocale(LC_CTYPE,"");
+  utf8mode = (strcmp(nl_langinfo(CODESET),"UTF-8")==0);
+  if (temp){
+    mylocale = dupstr(temp);
+    temp = strchr(mylocale,'.');
+    if (temp)
+      *temp = 0;
+  } else
+    mylocale = DEFAULTLOCALE;
+}
+
+#else
+
+void
+checklocale()
+{
+  char *temp;
+  temp = getenv("LC_CTYPE");
+  if (!temp)
+    temp = getenv("LANG");
+  if (!temp)
+    mylocale = DEFAULTLOCALE;
+  else {
+    mylocale = dupstr(temp);  
+    temp = strchr(mylocale,'.');
+    if (temp)
+      *temp = 0;
+  }
+}
+
+#endif
 
 int
 main(int argc, char **argv)
@@ -2625,7 +2730,7 @@ main(int argc, char **argv)
    int unitcount=0, prefixcount=0, funccount=0;   /* for counting units */
 
 #ifdef READLINE
-#  if RL_READLINE_VERSION >= 0x0402 
+#  if RL_READLINE_VERSION > 0x0402 
       rl_completion_entry_function = (rl_compentry_func_t *)completeunits;
 #  else
       rl_completion_entry_function = (Function *)completeunits;
@@ -2652,9 +2757,7 @@ main(int argc, char **argv)
      unitsfiles[unitcount+1] = 0;
    }
 
-   mylocale = getenv("LOCALE");
-   if (!mylocale)
-     mylocale = DEFAULTLOCALE;
+   checklocale();
 
    for(unitfileptr=unitsfiles;*unitfileptr;unitfileptr++){      
      readerr = readunits(*unitfileptr, stderr, &unitcount, &prefixcount, 
@@ -2678,7 +2781,7 @@ main(int argc, char **argv)
    }
 
    if (!interactive) {
-      if (funcval = isfunction(havestr)){
+      if ((funcval = isfunction(havestr))){
 	showfuncdefinition(funcval);
   	exit(0);
       }
@@ -2688,7 +2791,7 @@ main(int argc, char **argv)
          showdefinition(havestr,&have);
          exit(0);
       }
-      if (funcval = isfunction(wantstr)){
+      if ((funcval = isfunction(wantstr))){
          if (showfunc(havestr, &have, funcval))
 	   exit(1);
 	 else
@@ -2710,7 +2813,7 @@ main(int argc, char **argv)
 	 } while (isblankstr(havestr) || ishelpquery(havestr,0) ||
 		  (isfunction(havestr)==0 
 		  && processunit(&have, havestr, queryhave, POINT)));
-         if (funcval = isfunction(havestr)){
+         if ((funcval = isfunction(havestr))){
 	   showfuncdefinition(funcval);
 	   continue;
 	 }
@@ -2728,7 +2831,7 @@ main(int argc, char **argv)
 		  && processunit(&want, wantstr, querywant, POINT));
          if (isblankstr(wantstr))
            showdefinition(havestr,&have);
-         else if (funcval = isfunction(wantstr))
+         else if ((funcval = isfunction(wantstr)))
            showfunc(havestr, &have, funcval);
 	 else {
            showanswer(havestr,&have,wantstr, &want);
